@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2016  Gonzalo Exequiel Pedone
+ * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,27 +27,9 @@ class AkMultimediaSourceElementPrivate;
 
 typedef QSharedPointer<AkMultimediaSourceElement> AkMultimediaSourceElementPtr;
 
-class AkMultimediaSourceElement: public AkElement
+class AKCOMMONS_EXPORT AkMultimediaSourceElement: public AkElement
 {
     Q_OBJECT
-    Q_PROPERTY(QStringList medias
-               READ medias
-               NOTIFY mediasChanged)
-    Q_PROPERTY(QString media
-               READ media
-               WRITE setMedia
-               RESET resetMedia
-               NOTIFY mediaChanged)
-    Q_PROPERTY(QList<int> streams
-               READ streams
-               WRITE setStreams
-               RESET resetStreams
-               NOTIFY streamsChanged)
-    Q_PROPERTY(bool loop
-               READ loop
-               WRITE setLoop
-               RESET resetLoop
-               NOTIFY loopChanged)
 
     public:
         AkMultimediaSourceElement(QObject *parent=NULL);
@@ -65,13 +47,7 @@ class AkMultimediaSourceElement: public AkElement
     private:
         AkMultimediaSourceElementPrivate *d;
 
-    signals:
-        void mediasChanged(const QStringList &medias);
-        void mediaChanged(const QString &media);
-        void streamsChanged(const QList<int> &streams);
-        void loopChanged(bool loop);
-
-    public slots:
+    public Q_SLOTS:
         virtual void setMedia(const QString &media);
         virtual void setStreams(const QList<int> &streams);
         virtual void setLoop(bool loop);
