@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2016  Gonzalo Exequiel Pedone
+ * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,12 @@ typedef QMap<QFont::HintingPreference, QString> HintingPreferenceToStr;
 
 inline HintingPreferenceToStr initHintingPreferenceToStr()
 {
-    HintingPreferenceToStr hintingPreferenceToStr;
-    hintingPreferenceToStr[QFont::PreferDefaultHinting] = "PreferDefaultHinting";
-    hintingPreferenceToStr[QFont::PreferNoHinting] = "PreferNoHinting";
-    hintingPreferenceToStr[QFont::PreferVerticalHinting] = "PreferVerticalHinting";
-    hintingPreferenceToStr[QFont::PreferFullHinting] = "PreferFullHinting";
+    static const HintingPreferenceToStr hintingPreferenceToStr = {
+        {QFont::PreferDefaultHinting , "PreferDefaultHinting" },
+        {QFont::PreferNoHinting      , "PreferNoHinting"      },
+        {QFont::PreferVerticalHinting, "PreferVerticalHinting"},
+        {QFont::PreferFullHinting    , "PreferFullHinting"    }
+    };
 
     return hintingPreferenceToStr;
 }
@@ -40,20 +41,21 @@ typedef QMap<QFont::StyleStrategy, QString> StyleStrategyToStr;
 
 inline StyleStrategyToStr initStyleStrategyToStr()
 {
-    StyleStrategyToStr styleStrategyToStr;
-    styleStrategyToStr[QFont::PreferDefault] = "PreferDefault";
-    styleStrategyToStr[QFont::PreferBitmap] = "PreferBitmap";
-    styleStrategyToStr[QFont::PreferDevice] = "PreferDevice";
-    styleStrategyToStr[QFont::PreferOutline] = "PreferOutline";
-    styleStrategyToStr[QFont::ForceOutline] = "ForceOutline";
-    styleStrategyToStr[QFont::PreferMatch] = "PreferMatch";
-    styleStrategyToStr[QFont::PreferQuality] = "PreferQuality";
-    styleStrategyToStr[QFont::PreferAntialias] = "PreferAntialias";
-    styleStrategyToStr[QFont::NoAntialias] = "NoAntialias";
-    styleStrategyToStr[QFont::OpenGLCompatible] = "OpenGLCompatible";
-    styleStrategyToStr[QFont::ForceIntegerMetrics] = "ForceIntegerMetrics";
-    styleStrategyToStr[QFont::NoSubpixelAntialias] = "NoSubpixelAntialias";
-    styleStrategyToStr[QFont::NoFontMerging] = "NoFontMerging";
+    static const StyleStrategyToStr styleStrategyToStr = {
+        {QFont::PreferDefault      , "PreferDefault"      },
+        {QFont::PreferBitmap       , "PreferBitmap"       },
+        {QFont::PreferDevice       , "PreferDevice"       },
+        {QFont::PreferOutline      , "PreferOutline"      },
+        {QFont::ForceOutline       , "ForceOutline"       },
+        {QFont::PreferMatch        , "PreferMatch"        },
+        {QFont::PreferQuality      , "PreferQuality"      },
+        {QFont::PreferAntialias    , "PreferAntialias"    },
+        {QFont::NoAntialias        , "NoAntialias"        },
+        {QFont::OpenGLCompatible   , "OpenGLCompatible"   },
+        {QFont::ForceIntegerMetrics, "ForceIntegerMetrics"},
+        {QFont::NoSubpixelAntialias, "NoSubpixelAntialias"},
+        {QFont::NoFontMerging      , "NoFontMerging"      }
+    };
 
     return styleStrategyToStr;
 }
@@ -132,7 +134,7 @@ QObject *MatrixElement::controlInterface(QQmlEngine *engine, const QString &cont
 
     // Create a context for the plugin.
     QQmlContext *context = new QQmlContext(engine->rootContext());
-    context->setContextProperty("Matrix", (QObject *) this);
+    context->setContextProperty("Matrix", const_cast<QObject *>(qobject_cast<const QObject *>(this)));
     context->setContextProperty("controlId", this->objectName());
 
     // Create an item with the plugin context.
@@ -220,7 +222,7 @@ QSize MatrixElement::fontSize(const QString &chrTable, const QFont &font) const
     int width = -1;
     int height = -1;
 
-    foreach (QChar chr, chrTable) {
+    for (const QChar &chr: chrTable) {
         QSize size = metrics.size(Qt::TextSingleLine, chr);
 
         if (size.width() > width)
@@ -253,14 +255,16 @@ QImage MatrixElement::drawChar(const QChar &chr, const QFont &font,
 
 int MatrixElement::imageWeight(const QImage &image) const
 {
-    int fontArea = image.width() * image.height();
-    const QRgb *imageBits = (const QRgb *) image.constBits();
     int weight = 0;
 
-    for (int i = 0; i < fontArea; i++)
-        weight += qGray(imageBits[i]);
+    for (int y = 0; y < image.height(); y++) {
+        const QRgb *imageLine = reinterpret_cast<const QRgb *>(image.constScanLine(y));
 
-    weight /= fontArea;
+        for (int x = 0; x < image.width(); x++)
+            weight += qGray(imageLine[x]);
+    }
+
+    weight /= image.width() * image.height();
 
     return weight;
 }
@@ -447,7 +451,7 @@ void MatrixElement::setMaxDropLength(int maxDropLength)
 
 void MatrixElement::setMinSpeed(qreal minSpeed)
 {
-    if (this->m_minSpeed == minSpeed)
+    if (qFuzzyCompare(this->m_minSpeed, minSpeed))
         return;
 
     QMutexLocker(&this->m_mutex);
@@ -457,7 +461,7 @@ void MatrixElement::setMinSpeed(qreal minSpeed)
 
 void MatrixElement::setMaxSpeed(qreal maxSpeed)
 {
-    if (this->m_maxSpeed == maxSpeed)
+    if (qFuzzyCompare(this->m_maxSpeed, maxSpeed))
         return;
 
     QMutexLocker(&this->m_mutex);
@@ -574,7 +578,7 @@ AkPacket MatrixElement::iStream(const AkPacket &packet)
     }
 
     QImage textImage = src.scaled(textWidth, textHeight);
-    QRgb *textImageBits = (QRgb *) textImage.bits();
+    QRgb *textImageBits = reinterpret_cast<QRgb *>(textImage.bits());
     int textArea = textImage.width() * textImage.height();
     QPainter painter;
 
@@ -607,7 +611,7 @@ void MatrixElement::updateCharTable()
     for (int i = 0; i < 256; i++)
         colorTable[i] = qRgb(i, i, i);
 
-    foreach (QChar chr, this->m_charTable) {
+    for (const QChar &chr: this->m_charTable) {
         QImage image = drawChar(chr,
                                 this->m_font,
                                 this->m_fontSize,
