@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2016  Gonzalo Exequiel Pedone
+ * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ QObject *PhotocopyElement::controlInterface(QQmlEngine *engine,
 
     // Create a context for the plugin.
     QQmlContext *context = new QQmlContext(engine->rootContext());
-    context->setContextProperty("Photocopy", (QObject *) this);
+    context->setContextProperty("Photocopy", const_cast<QObject *>(qobject_cast<const QObject *>(this)));
     context->setContextProperty("controlId", this->objectName());
 
     // Create an item with the plugin context.
@@ -79,7 +79,7 @@ qreal PhotocopyElement::contrast() const
 
 void PhotocopyElement::setBrightness(qreal brightness)
 {
-    if (this->m_brightness == brightness)
+    if (qFuzzyCompare(this->m_brightness, brightness))
         return;
 
     this->m_brightness = brightness;
@@ -88,7 +88,7 @@ void PhotocopyElement::setBrightness(qreal brightness)
 
 void PhotocopyElement::setContrast(qreal contrast)
 {
-    if (this->m_contrast == contrast)
+    if (qFuzzyCompare(this->m_contrast, contrast))
         return;
 
     this->m_contrast = contrast;
@@ -113,27 +113,28 @@ AkPacket PhotocopyElement::iStream(const AkPacket &packet)
         return AkPacket();
 
     src = src.convertToFormat(QImage::Format_ARGB32);
-    int videoArea = src.width() * src.height();
     QImage oFrame(src.size(), src.format());
 
-    QRgb *srcBits = (QRgb *) src.bits();
-    QRgb *destBits = (QRgb *) oFrame.bits();
+    for (int y = 0; y < src.height(); y++) {
+        const QRgb *srcLine = reinterpret_cast<const QRgb *>(src.constScanLine(y));
+        QRgb *dstLine = reinterpret_cast<QRgb *>(oFrame.scanLine(y));
 
-    for (int i = 0; i < videoArea; i++) {
-        int r = qRed(srcBits[i]);
-        int g = qGreen(srcBits[i]);
-        int b = qBlue(srcBits[i]);
+        for (int x = 0; x < src.width(); x++) {
+            int r = qRed(srcLine[x]);
+            int g = qGreen(srcLine[x]);
+            int b = qBlue(srcLine[x]);
 
-        //desaturate
-        int luma = this->rgbToLuma(r, g, b);
+            //desaturate
+            int luma = this->rgbToLuma(r, g, b);
 
-        //compute sigmoidal transfer
-        qreal val = luma / 255.0;
-        val = 255.0 / (1 + exp(this->m_contrast * (0.5 - val)));
-        val = val * this->m_brightness;
-        luma = qBound(0.0, val, 255.0);
+            //compute sigmoidal transfer
+            qreal val = luma / 255.0;
+            val = 255.0 / (1 + exp(this->m_contrast * (0.5 - val)));
+            val = val * this->m_brightness;
+            luma = int(qBound(0.0, val, 255.0));
 
-        destBits[i] = qRgba(luma, luma, luma, qAlpha(srcBits[i]));
+            dstLine[x] = qRgba(luma, luma, luma, qAlpha(srcLine[x]));
+        }
     }
 
     AkPacket oPacket = AkUtils::imageToPacket(oFrame, packet);
