@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2016  Gonzalo Exequiel Pedone
+ * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,44 +20,83 @@
 #include <akutils.h>
 
 #include "videocaptureelement.h"
+#include "videocaptureglobals.h"
 
 #define PAUSE_TIMEOUT 500
 
+#ifdef Q_OS_WIN32
+#include <combaseapi.h>
+
+inline QStringList initMirrorFormats()
+{
+    QStringList mirrorFormats = {"RGB3", "RGB4", "RGBP", "RGBO"};
+
+    return mirrorFormats;
+}
+
+Q_GLOBAL_STATIC_WITH_ARGS(QStringList, mirrorFormats, (initMirrorFormats()))
+
+inline QStringList initSwapRgbFormats()
+{
+    QStringList swapRgbFormats = {"RGB3", "YV12"};
+
+    return swapRgbFormats;
+}
+
+Q_GLOBAL_STATIC_WITH_ARGS(QStringList, swapRgbFormats, (initSwapRgbFormats()))
+#endif
+
+Q_GLOBAL_STATIC(VideoCaptureGlobals, globalVideoCapture)
+
+template<typename T>
+inline QSharedPointer<T> ptr_init(QObject *obj=nullptr)
+{
+    if (!obj)
+        return QSharedPointer<T>(new T());
+
+    return QSharedPointer<T>(static_cast<T *>(obj));
+}
+
+template <typename T>
+inline void waitLoop(const QFuture<T> &loop)
+{
+    while (!loop.isFinished()) {
+        auto eventDispatcher = QThread::currentThread()->eventDispatcher();
+
+        if (eventDispatcher)
+            eventDispatcher->processEvents(QEventLoop::AllEvents);
+    }
+}
+
 VideoCaptureElement::VideoCaptureElement():
-    AkMultimediaSourceElement()
+    AkMultimediaSourceElement(),
+    m_convertVideo(ptr_init<ConvertVideo>()),
+    m_capture(ptr_init<Capture>())
 {
     this->m_runCameraLoop = false;
     this->m_pause = false;
+    this->m_mirror = false;
+    this->m_swapRgb = false;
 
-    QObject::connect(&this->m_capture,
-                     &Capture::error,
+    QObject::connect(globalVideoCapture,
+                     SIGNAL(codecLibChanged(const QString &)),
                      this,
-                     &VideoCaptureElement::error);
-    QObject::connect(&this->m_capture,
-                     &Capture::webcamsChanged,
+                     SIGNAL(codecLibChanged(const QString &)));
+    QObject::connect(globalVideoCapture,
+                     SIGNAL(codecLibChanged(const QString &)),
                      this,
-                     &VideoCaptureElement::mediasChanged);
-    QObject::connect(&this->m_capture,
-                     &Capture::deviceChanged,
+                     SLOT(codecLibUpdated(const QString &)));
+    QObject::connect(globalVideoCapture,
+                     SIGNAL(captureLibChanged(const QString &)),
                      this,
-                     &VideoCaptureElement::mediaChanged);
-    QObject::connect(&this->m_capture,
-                     &Capture::imageControlsChanged,
+                     SIGNAL(captureLibChanged(const QString &)));
+    QObject::connect(globalVideoCapture,
+                     SIGNAL(captureLibChanged(const QString &)),
                      this,
-                     &VideoCaptureElement::imageControlsChanged);
-    QObject::connect(&this->m_capture,
-                     &Capture::cameraControlsChanged,
-                     this,
-                     &VideoCaptureElement::cameraControlsChanged);
-    QObject::connect(&this->m_capture,
-                     &Capture::streamsChanged,
-                     this,
-                     &VideoCaptureElement::streamsChanged);
-    QObject::connect(&this->m_convertVideo,
-                     &ConvertVideo::frameReady,
-                     this,
-                     &VideoCaptureElement::frameReady,
-                     Qt::DirectConnection);
+                     SLOT(captureLibUpdated(const QString &)));
+
+    this->codecLibUpdated(globalVideoCapture->codecLib());
+    this->captureLibUpdated(globalVideoCapture->captureLib());
 }
 
 VideoCaptureElement::~VideoCaptureElement()
@@ -84,7 +123,7 @@ QObject *VideoCaptureElement::controlInterface(QQmlEngine *engine, const QString
 
     // Create a context for the plugin.
     QQmlContext *context = new QQmlContext(engine->rootContext());
-    context->setContextProperty("VideoCapture", (QObject *) this);
+    context->setContextProperty("VideoCapture", const_cast<QObject *>(qobject_cast<const QObject *>(this)));
     context->setContextProperty("controlId", controlId);
 
     // Create an item with the plugin context.
@@ -103,22 +142,22 @@ QObject *VideoCaptureElement::controlInterface(QQmlEngine *engine, const QString
 
 QStringList VideoCaptureElement::medias() const
 {
-    return this->m_capture.webcams();
+    return this->m_capture->webcams();
 }
 
 QString VideoCaptureElement::media() const
 {
-    return this->m_capture.device();
+    return this->m_capture->device();
 }
 
 QList<int> VideoCaptureElement::streams() const
 {
-    return this->m_capture.streams();
+    return this->m_capture->streams();
 }
 
 QList<int> VideoCaptureElement::listTracks(const QString &mimeType)
 {
-    return this->m_capture.listTracks(mimeType);
+    return this->m_capture->listTracks(mimeType);
 }
 
 int VideoCaptureElement::defaultStream(const QString &mimeType) const
@@ -131,12 +170,12 @@ int VideoCaptureElement::defaultStream(const QString &mimeType) const
 
 QString VideoCaptureElement::description(const QString &media) const
 {
-    return this->m_capture.description(media);
+    return this->m_capture->description(media);
 }
 
 AkCaps VideoCaptureElement::caps(int stream) const
 {
-    QVariantList streams = this->m_capture.caps(this->m_capture.device());
+    QVariantList streams = this->m_capture->caps(this->m_capture->device());
     AkCaps caps = streams.value(stream).value<AkCaps>();
 
     if (!caps)
@@ -153,55 +192,72 @@ AkCaps VideoCaptureElement::caps(int stream) const
     return videoCaps;
 }
 
+AkCaps VideoCaptureElement::rawCaps(int stream) const
+{
+    QVariantList streams = this->m_capture->caps(this->m_capture->device());
+
+    return streams.value(stream).value<AkCaps>();
+}
+
 QStringList VideoCaptureElement::listCapsDescription() const
 {
     QStringList capsDescriptions;
-    QVariantList streams = this->m_capture.caps(this->m_capture.device());
+    QVariantList streams = this->m_capture->caps(this->m_capture->device());
 
-    foreach (QVariant caps, streams)
-        capsDescriptions << this->m_capture.capsDescription(caps.value<AkCaps>());
+    for (const QVariant &caps: streams)
+        capsDescriptions << this->m_capture->capsDescription(caps.value<AkCaps>());
 
     return capsDescriptions;
 }
 
 QString VideoCaptureElement::ioMethod() const
 {
-    return this->m_capture.ioMethod();
+    return this->m_capture->ioMethod();
 }
 
 int VideoCaptureElement::nBuffers() const
 {
-    return this->m_capture.nBuffers();
+    return this->m_capture->nBuffers();
+}
+
+QString VideoCaptureElement::codecLib() const
+{
+    return globalVideoCapture->codecLib();
+}
+
+QString VideoCaptureElement::captureLib() const
+{
+    return globalVideoCapture->captureLib();
 }
 
 QVariantList VideoCaptureElement::imageControls() const
 {
-    return this->m_capture.imageControls();
+    return this->m_capture->imageControls();
 }
 
-bool VideoCaptureElement::setImageControls(const QVariantMap &imageControls) const
+bool VideoCaptureElement::setImageControls(const QVariantMap &imageControls)
 {
-    return this->m_capture.setImageControls(imageControls);
+    return this->m_capture->setImageControls(imageControls);
 }
 
-bool VideoCaptureElement::resetImageControls() const
+bool VideoCaptureElement::resetImageControls()
 {
-    return this->m_capture.resetImageControls();
+    return this->m_capture->resetImageControls();
 }
 
 QVariantList VideoCaptureElement::cameraControls() const
 {
-    return this->m_capture.cameraControls();
+    return this->m_capture->cameraControls();
 }
 
-bool VideoCaptureElement::setCameraControls(const QVariantMap &cameraControls) const
+bool VideoCaptureElement::setCameraControls(const QVariantMap &cameraControls)
 {
-    return this->m_capture.setCameraControls(cameraControls);
+    return this->m_capture->setCameraControls(cameraControls);
 }
 
-bool VideoCaptureElement::resetCameraControls() const
+bool VideoCaptureElement::resetCameraControls()
 {
-    return this->m_capture.resetCameraControls();
+    return this->m_capture->resetCameraControls();
 }
 
 void VideoCaptureElement::cameraLoop(VideoCaptureElement *captureElement)
@@ -211,7 +267,9 @@ void VideoCaptureElement::cameraLoop(VideoCaptureElement *captureElement)
     CoInitializeEx(NULL, COINIT_MULTITHREADED);
 #endif
 
-    if (captureElement->m_capture.init()) {
+    bool initConvert = true;
+
+    if (captureElement->m_capture->init()) {
         while (captureElement->m_runCameraLoop) {
             if (captureElement->m_pause) {
                 QThread::msleep(PAUSE_TIMEOUT);
@@ -219,15 +277,31 @@ void VideoCaptureElement::cameraLoop(VideoCaptureElement *captureElement)
                 continue;
             }
 
-            AkPacket packet = captureElement->m_capture.readFrame();
+            AkPacket packet = captureElement->m_capture->readFrame();
 
             if (!packet)
                 continue;
 
-            captureElement->m_convertVideo.packetEnqueue(packet);
+            if (initConvert) {
+                AkCaps caps = packet.caps();
+
+#ifdef Q_OS_WIN32
+                QString fourcc = caps.property("fourcc").toString();
+                captureElement->m_mirror = mirrorFormats->contains(fourcc);
+                captureElement->m_swapRgb = swapRgbFormats->contains(fourcc);
+#endif
+
+                if (!captureElement->m_convertVideo->init(caps))
+                    break;
+
+                initConvert = false;
+            }
+
+            captureElement->m_convertVideo->packetEnqueue(packet);
         }
 
-        captureElement->m_capture.uninit();
+        captureElement->m_convertVideo->uninit();
+        captureElement->m_capture->uninit();
     }
 
 #ifdef Q_OS_WIN32
@@ -238,7 +312,7 @@ void VideoCaptureElement::cameraLoop(VideoCaptureElement *captureElement)
 
 void VideoCaptureElement::setMedia(const QString &media)
 {
-    this->m_capture.setDevice(media);
+    this->m_capture->setDevice(media);
 }
 
 void VideoCaptureElement::setStreams(const QList<int> &streams)
@@ -246,7 +320,7 @@ void VideoCaptureElement::setStreams(const QList<int> &streams)
     bool running = this->m_runCameraLoop;
     this->setState(AkElement::ElementStateNull);
 
-    this->m_capture.setStreams(streams);
+    this->m_capture->setStreams(streams);
 
     if (running)
         this->setState(AkElement::ElementStatePlaying);
@@ -254,37 +328,57 @@ void VideoCaptureElement::setStreams(const QList<int> &streams)
 
 void VideoCaptureElement::setIoMethod(const QString &ioMethod)
 {
-    this->m_capture.setIoMethod(ioMethod);
+    this->m_capture->setIoMethod(ioMethod);
 }
 
 void VideoCaptureElement::setNBuffers(int nBuffers)
 {
-    this->m_capture.setNBuffers(nBuffers);
+    this->m_capture->setNBuffers(nBuffers);
+}
+
+void VideoCaptureElement::setCodecLib(const QString &codecLib)
+{
+    globalVideoCapture->setCodecLib(codecLib);
+}
+
+void VideoCaptureElement::setCaptureLib(const QString &captureLib)
+{
+    globalVideoCapture->setCaptureLib(captureLib);
 }
 
 void VideoCaptureElement::resetMedia()
 {
-    this->m_capture.resetDevice();
+    this->m_capture->resetDevice();
 }
 
 void VideoCaptureElement::resetStreams()
 {
-    this->m_capture.resetStreams();
+    this->m_capture->resetStreams();
 }
 
 void VideoCaptureElement::resetIoMethod()
 {
-    this->m_capture.resetIoMethod();
+    this->m_capture->resetIoMethod();
 }
 
 void VideoCaptureElement::resetNBuffers()
 {
-    this->m_capture.resetNBuffers();
+    this->m_capture->resetNBuffers();
+}
+
+void VideoCaptureElement::resetCodecLib()
+{
+    globalVideoCapture->resetCodecLib();
+}
+
+void VideoCaptureElement::resetCaptureLib()
+{
+    globalVideoCapture->resetCaptureLib();
 }
 
 void VideoCaptureElement::reset()
 {
-    this->m_capture.reset();
+    this->m_capture->reset();
 }
 
 bool VideoCaptureElement::setState(AkElement::ElementState state)
@@ -295,17 +389,6 @@ bool VideoCaptureElement::setState(AkElement::ElementState state)
     case AkElement::ElementStateNull: {
         switch (state) {
         case AkElement::ElementStatePaused: {
-            QList<int> streams = this->m_capture.streams();
-
-            if (streams.isEmpty())
-                return false;
-
-            QVariantList supportedCaps = this->m_capture.caps(this->m_capture.device());
-            AkCaps caps = supportedCaps.value(streams[0]).value<AkCaps>();
-
-            if (!this->m_convertVideo.init(caps))
-                return false;
-
             this->m_pause = true;
             this->m_runCameraLoop = true;
             this->m_cameraLoopResult = QtConcurrent::run(&this->m_threadPool, this->cameraLoop, this);
@@ -313,24 +396,13 @@ bool VideoCaptureElement::setState(AkElement::ElementState state)
             return AkElement::setState(state);
         }
         case AkElement::ElementStatePlaying: {
-            QList<int> streams = this->m_capture.streams();
-
-            if (streams.isEmpty())
-                return false;
-
-            QVariantList supportedCaps = this->m_capture.caps(this->m_capture.device());
-            AkCaps caps = supportedCaps.value(streams[0]).value<AkCaps>();
-
-            if (!this->m_convertVideo.init(caps))
-                return false;
-
             this->m_pause = false;
             this->m_runCameraLoop = true;
             this->m_cameraLoopResult = QtConcurrent::run(&this->m_threadPool, this->cameraLoop, this);
 
             return AkElement::setState(state);
         }
-        default:
+        case AkElement::ElementStateNull:
             break;
         }
 
@@ -341,15 +413,14 @@ bool VideoCaptureElement::setState(AkElement::ElementState state)
         case AkElement::ElementStateNull:
             this->m_pause = false;
             this->m_runCameraLoop = false;
-            this->m_cameraLoopResult.waitForFinished();
-            this->m_convertVideo.uninit();
+            waitLoop(this->m_cameraLoopResult);
 
             return AkElement::setState(state);
         case AkElement::ElementStatePlaying:
             this->m_pause = false;
 
             return AkElement::setState(state);
-        default:
+        case AkElement::ElementStatePaused:
             break;
         }
 
@@ -357,24 +428,22 @@ bool VideoCaptureElement::setState(AkElement::ElementState state)
     }
     case AkElement::ElementStatePlaying: {
         switch (state) {
-        case AkElement::ElementStateNull:
+        case AkElement::ElementStateNull: {
             this->m_runCameraLoop = false;
-            this->m_cameraLoopResult.waitForFinished();
-            this->m_convertVideo.uninit();
+            waitLoop(this->m_cameraLoopResult);
 
             return AkElement::setState(state);
+        }
         case AkElement::ElementStatePaused:
             this->m_pause = true;
 
             return AkElement::setState(state);
-        default:
+        case AkElement::ElementStatePlaying:
             break;
         }
 
         break;
     }
-    default:
-        break;
     }
 
     return false;
@@ -382,11 +451,87 @@ bool VideoCaptureElement::setState(AkElement::ElementState state)
 
 void VideoCaptureElement::frameReady(const AkPacket &packet)
 {
-#if defined(Q_OS_WIN32)
-    QImage oImage = AkUtils::packetToImage(packet).mirrored().rgbSwapped();
+#ifdef Q_OS_WIN32
+    if (this->m_mirror || this->m_swapRgb) {
+        QImage oImage = AkUtils::packetToImage(packet);
 
-    emit this->oStream(AkUtils::imageToPacket(oImage, packet));
-#else
-    emit this->oStream(packet);
+        if (this->m_mirror)
+            oImage = oImage.mirrored();
+
+        if (this->m_swapRgb)
+            oImage = oImage.rgbSwapped();
+
+        emit this->oStream(AkUtils::imageToPacket(oImage, packet));
+    } else
 #endif
+        emit this->oStream(packet);
+}
+
+void VideoCaptureElement::codecLibUpdated(const QString &codecLib)
+{
+    auto state = this->state();
+    this->setState(AkElement::ElementStateNull);
+
+    this->m_mutexLib.lock();
+
+    this->m_convertVideo =
+            ptr_init<ConvertVideo>(this->loadSubModule("VideoCapture", codecLib));
+
+    QObject::connect(this->m_convertVideo.data(),
+                     &ConvertVideo::frameReady,
+                     this,
+                     &VideoCaptureElement::frameReady,
+                     Qt::DirectConnection);
+
+    this->m_mutexLib.unlock();
+
+    this->setState(state);
+}
+
+void VideoCaptureElement::captureLibUpdated(const QString &captureLib)
+{
+    auto state = this->state();
+    this->setState(AkElement::ElementStateNull);
+
+    this->m_mutexLib.lock();
+
+    this->m_capture =
+            ptr_init<Capture>(this->loadSubModule("VideoCapture", captureLib));
+
+    this->m_mutexLib.unlock();
+
+    QObject::connect(this->m_capture.data(),
+                     &Capture::error,
+                     this,
+                     &VideoCaptureElement::error);
+    QObject::connect(this->m_capture.data(),
+                     &Capture::webcamsChanged,
+                     this,
+                     &VideoCaptureElement::mediasChanged);
+    QObject::connect(this->m_capture.data(),
+                     &Capture::deviceChanged,
+                     this,
+                     &VideoCaptureElement::mediaChanged);
+    QObject::connect(this->m_capture.data(),
+                     &Capture::imageControlsChanged,
+                     this,
+                     &VideoCaptureElement::imageControlsChanged);
+    QObject::connect(this->m_capture.data(),
+                     &Capture::cameraControlsChanged,
+                     this,
+                     &VideoCaptureElement::cameraControlsChanged);
+    QObject::connect(this->m_capture.data(),
+                     &Capture::streamsChanged,
+                     this,
+                     &VideoCaptureElement::streamsChanged);
+
+    emit this->mediasChanged(this->medias());
+    emit this->streamsChanged(this->streams());
+
+    auto medias = this->medias();
+
+    if (!medias.isEmpty())
+        this->setMedia(medias.first());
+
+    this->setState(state);
 }
